@@ -15,24 +15,20 @@
 # SOFTWARE.
 from __future__ import annotations
 
+import time
 import json
 import os
 from typing import TYPE_CHECKING
 from zipfile import ZipFile
 
-import rdflib
-from filelock import FileLock
-from rdflib import RDF, ConjunctiveGraph, Graph, Namespace, URIRef
-from SPARQLWrapper import JSONLD, SPARQLWrapper
+from rdflib import RDF, ConjunctiveGraph, Graph, URIRef
+from SPARQLWrapper import JSON, SPARQLWrapper
 
 from oc_ocdm.graph.graph_entity import GraphEntity
 from oc_ocdm.support.reporter import Reporter
-
+from oc_ocdm.support.support import build_graph_from_results
 if TYPE_CHECKING:
     from typing import Any, Dict, List, Optional, Set
-
-    from rdflib import term
-
     from oc_ocdm.graph.graph_set import GraphSet
 
 from pyshacl import validate
@@ -84,59 +80,59 @@ class Reader(object):
         return loaded_graph
 
     def _load_graph(self, file_path: str) -> ConjunctiveGraph:
-        formats: List[str] = ["json-ld", "rdfxml", "turtle", "trig", "nt11", "nquads"]
+        formats = ["json-ld", "rdfxml", "turtle", "trig", "nt11", "nquads"]
+        loaded_graph = ConjunctiveGraph()
 
-        loaded_graph: ConjunctiveGraph = ConjunctiveGraph()
-
-        errors: str = ""
-        for cur_format in formats:
+        if file_path.endswith('.zip'):
             try:
-                lock = FileLock(f"{file_path}.lock")
-                with lock:
-                    if file_path.endswith('.zip'):
-                        with ZipFile(file=file_path, mode="r") as archive:
-                            for zf_name in archive.namelist():
-                                f = archive.open(zf_name)
-                    else:
-                        f = open(file_path, 'rt', encoding='utf-8')
-                    if cur_format == "json-ld":
-                        json_ld_file: Any = json.load(f)
-                        if isinstance(json_ld_file, dict):
-                            json_ld_file: List[Any] = [json_ld_file]
-
-                        for json_ld_resource in json_ld_file:
-                            # Trick to force the use of a pre-loaded context if the format
-                            # specified is JSON-LD
-                            if "@context" in json_ld_resource:
-                                cur_context: str = json_ld_resource["@context"]
-                                if cur_context in self.context_map:
-                                    context_json: Any = self.context_map[cur_context]["@context"]
-                                    json_ld_resource["@context"] = context_json
-
-                            loaded_graph.parse(data=json.dumps(json_ld_resource, ensure_ascii=False),
-                                            format=cur_format)
-                    else:
-                        loaded_graph.parse(file=f, format=cur_format)
-                    f.close()
-                return loaded_graph
+                with ZipFile(file=file_path, mode="r") as archive:
+                    for zf_name in archive.namelist():
+                        with archive.open(zf_name) as f:
+                            if self._try_parse(loaded_graph, f, formats):
+                                return loaded_graph
             except Exception as e:
-                errors += f" | {e}"  # Try another format
+                raise IOError(f"Error opening or reading zip file '{file_path}': {e}")
+        else:
+            try:
+                with open(file_path, 'rt', encoding='utf-8') as f:
+                    if self._try_parse(loaded_graph, f, formats):
+                        return loaded_graph
+            except Exception as e:
+                raise IOError(f"Error opening or reading file '{file_path}': {e}")
 
-        raise IOError("1", f"It was impossible to handle the format used for storing the file '{file_path}'{errors}")
+        raise IOError(f"It was impossible to load the file '{file_path}' with supported formats.")
+
+    def _try_parse(self, graph: ConjunctiveGraph, file_obj, formats: List[str]) -> bool:
+        for cur_format in formats:
+            file_obj.seek(0)  # Reset file pointer to the beginning for each new attempt
+            try:
+                if cur_format == "json-ld":
+                    json_ld_file = json.load(file_obj)
+                    if isinstance(json_ld_file, dict):
+                        json_ld_file = [json_ld_file]
+                    for json_ld_resource in json_ld_file:
+                        if "@context" in json_ld_resource and json_ld_resource["@context"] in self.context_map:
+                            json_ld_resource["@context"] = self.context_map[json_ld_resource["@context"]]["@context"]
+                    data = json.dumps(json_ld_file, ensure_ascii=False)
+                    graph.parse(data=data, format=cur_format)
+                else:
+                    graph.parse(file=file_obj, format=cur_format)
+                return True  # Success, no need to try other formats
+            except Exception as e:
+                continue  # Try the next format
+        return False  # None of the formats succeeded
 
     @staticmethod
     def get_graph_from_subject(graph: Graph, subject: URIRef) -> Graph:
         g: Graph = Graph(identifier=graph.identifier)
-        for p, o in graph.predicate_objects(subject):
-            if isinstance(o, rdflib.term.Literal):
-                o = rdflib.term.Literal(lexical_or_value=str(o), datatype=None)
+        for p, o in graph.predicate_objects(subject, unique=True):
             g.add((subject, p, o))
         return g
 
     @staticmethod
     def _extract_subjects(graph: Graph) -> Set[URIRef]:
         subjects: Set[URIRef] = set()
-        for s in graph.subjects():
+        for s in graph.subjects(unique=True):
             subjects.add(s)
         return subjects
 
@@ -147,7 +143,7 @@ class Reader(object):
             sg.parse(os.path.join('oc_ocdm', 'resources', 'shacle_closed.ttl'))
         else:
             sg.parse(os.path.join('oc_ocdm', 'resources', 'shacle.ttl'))
-        _, report_g, report = validate(graph,
+        _, report_g, _ = validate(graph,
             shacl_graph=sg,
             ont_graph=None,
             inference=None,
@@ -168,14 +164,15 @@ class Reader(object):
         return valid_graph
 
     @staticmethod
-    def import_entities_from_graph(g_set: GraphSet, graph: Graph, resp_agent: str,
+    def import_entities_from_graph(g_set: GraphSet, results: List[Dict], resp_agent: str,
                                    enable_validation: bool = False, closed: bool = False) -> List[GraphEntity]:
+        graph = build_graph_from_results(results)
         if enable_validation:
             reader = Reader()
             graph = reader.graph_validation(graph, closed)
         imported_entities: List[GraphEntity] = []
         for subject in Reader._extract_subjects(graph):
-            types: List[term] = []
+            types = []
             for o in graph.objects(subject, RDF.type):
                 types.append(o)
             # ReferenceAnnotation
@@ -227,16 +224,35 @@ class Reader(object):
     @staticmethod
     def import_entity_from_triplestore(g_set: GraphSet, ts_url: str, res: URIRef, resp_agent: str,
                                        enable_validation: bool = False) -> GraphEntity:
-        sparql: SPARQLWrapper = SPARQLWrapper(ts_url)
-        query: str = f"CONSTRUCT {{<{res}> ?p ?o}} WHERE {{<{res}> ?p ?o}}"
-        sparql.setQuery(query)
-        sparql.setMethod('GET')
-        sparql.setReturnFormat(JSONLD)
-        result: ConjunctiveGraph = sparql.query().convert()
-        if result is not None:
-            imported_entities: List[GraphEntity] = Reader.import_entities_from_graph(g_set, result,
-                                                                                     resp_agent, enable_validation)
-            if len(imported_entities) <= 0:
-                raise ValueError("The requested entity was not found or was not recognized as a proper OCDM entity.")
-            else:
-                return imported_entities[0]
\ No newline at end of file
+        query: str = f"SELECT ?s ?p ?o WHERE {{BIND (<{res}> AS ?s). ?s ?p ?o.}}"
+        attempt = 0
+        max_attempts = 3
+        wait_time = 5  # Initial wait time in seconds
+
+        while attempt < max_attempts:
+            try:
+                sparql: SPARQLWrapper = SPARQLWrapper(ts_url)
+                sparql.setQuery(query)
+                sparql.setMethod('GET')
+                sparql.setReturnFormat(JSON)
+                result = sparql.queryAndConvert()['results']['bindings']
+                
+                if result:
+                    imported_entities: List[GraphEntity] = Reader.import_entities_from_graph(g_set, result, resp_agent, enable_validation)
+                    if len(imported_entities) <= 0:
+                        raise ValueError("The requested entity was not found or was not recognized as a proper OCDM entity.")
+                    else:
+                        return imported_entities[0]
+
+            except Exception as e:
+                attempt += 1
+                if attempt < max_attempts:
+                    print(f"[3] Attempt {attempt} failed. Could not import entity due to communication problems: {e}")
+                    print(f"Retrying in {wait_time} seconds...")
+                    time.sleep(wait_time)
+                    wait_time *= 2  # Double the wait time for the next attempt
+                else:
+                    print(f"[3] All {max_attempts} attempts failed. Could not import entity due to communication problems: {e}")
+                    raise
+
+        raise Exception("Max attempts reached. Failed to import entity from triplestore.")
\ No newline at end of file
