@@ -19,10 +19,10 @@ import os
 import re
 from datetime import datetime
 from typing import TYPE_CHECKING
+from rdflib import URIRef, Graph
 
 if TYPE_CHECKING:
     from typing import Optional, List, Tuple, Match, Dict, Set
-    from rdflib import URIRef, Graph
     from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import BibliographicResource
     from oc_ocdm.graph.entities.bibliographic.responsible_agent import ResponsibleAgent
     from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole
@@ -168,8 +168,8 @@ def is_string_empty(string: str) -> bool:
 
 
 # Variable used in several functions
-entity_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?([1-9][0-9]*)$"
-prov_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?([1-9][0-9]*)/prov/([a-z][a-z])/([1-9][0-9]*)$"
+entity_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))$"
+prov_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))/prov/([a-z][a-z])/([1-9][0-9]*)$"
 
 
 def _get_match(regex: str, group: int, string: str) -> str:
@@ -259,7 +259,7 @@ def find_local_line_id(res: URIRef, n_file_item: int = 1) -> int:
 
 
 def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_split: int,
-               n_file_item: int, is_json: bool = True) -> Tuple[str, str]:
+               n_file_item: int, is_json: bool = True, process_id: int|str = None) -> Tuple[str, str]:
     """
     This function is responsible for looking for the correct JSON file that contains the data related to the
     resource identified by the variable 'string_iri'. This search takes into account the organisation in
@@ -267,12 +267,13 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
     In case no supplier prefix is specified, the 'default_dir' (usually set to "_") is used instead.
     """
     string_iri: str = str(res)
+    process_id_str: str = f"_{process_id}" if process_id else ""
 
     if is_dataset(res):
         cur_dir_path: str = (base_dir + re.sub(r"^%s(.*)$" % base_iri, r"\1", string_iri))[:-1]
         # In case of dataset, the file path is different from regular files, e.g.
         # /corpus/br/index.json
-        cur_file_path: str = cur_dir_path + os.sep + "index.json"
+        cur_file_path: str = cur_dir_path + os.sep + "index" + process_id_str + ".json"
     else:
         cur_number: int = get_resource_number(res)
 
@@ -307,7 +308,7 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
 
                 cur_dir_path: str = base_dir + subj_short_name + os.sep + sub_folder + \
                     os.sep + str(cur_split) + os.sep + str(cur_file_split) + os.sep + "prov"
-                cur_file_path: str = cur_dir_path + os.sep + short_name + file_extension
+                cur_file_path: str = cur_dir_path + os.sep + short_name + process_id_str + file_extension
             else:  # regular bibliographic entity
                 short_name: str = get_short_name(res)
                 sub_folder: str = get_prefix(res)
@@ -318,7 +319,7 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
                     sub_folder = "_"  # enforce default value
 
                 cur_dir_path: str = base_dir + short_name + os.sep + sub_folder + os.sep + str(cur_split)
-                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + file_extension
+                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
         # Enter here if no split is needed
         elif dir_split == 0:
             if "/prov/" in string_iri:
@@ -333,7 +334,7 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
 
                 cur_dir_path: str = base_dir + subj_short_name + os.sep + sub_folder + \
                     os.sep + str(cur_file_split) + os.sep + "prov"
-                cur_file_path: str = cur_dir_path + os.sep + short_name + file_extension
+                cur_file_path: str = cur_dir_path + os.sep + short_name + process_id_str + file_extension
             else:
                 short_name: str = get_short_name(res)
                 sub_folder: str = get_prefix(res)
@@ -344,7 +345,7 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
                     sub_folder = "_"  # enforce default value
 
                 cur_dir_path: str = base_dir + short_name + os.sep + sub_folder
-                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + file_extension
+                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
         # Enter here if the data is about a provenance agent, e.g. /corpus/prov/
         else:
             short_name: str = get_short_name(res)
@@ -353,7 +354,7 @@ def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_
             file_extension: str = '.json' if is_json else '.nq'
 
             cur_dir_path: str = base_dir + short_name
-            cur_file_path: str = cur_dir_path + os.sep + prefix + count + file_extension
+            cur_file_path: str = cur_dir_path + os.sep + prefix + count + process_id_str + file_extension
 
     return cur_dir_path, cur_file_path
 
@@ -361,6 +362,20 @@ def has_supplier_prefix(res: URIRef, base_iri: str) -> bool:
     string_iri: str = str(res)
     return re.search(r"^%s[a-z][a-z]/0" % base_iri, string_iri) is not None
 
+def build_graph_from_results(results: List[Dict]) -> Graph:
+    graph = Graph()
+    for triple in results:
+        s = URIRef(triple['s']['value'])
+        p = URIRef(triple['p']['value'])
+        if triple['o']['type'] == 'uri':
+            o = URIRef(triple['o']['value'])
+        else:
+            datatype = triple['o'].get('datatype', None)
+            datatype = URIRef(datatype) if datatype is not None else None
+            o = Literal(triple['o']['value'], datatype=datatype)
+        graph.add((s, p, o))
+    return graph
+
 
 def is_dataset(res: URIRef) -> bool:
     string_iri: str = str(res)
